package com.ibm.bbp.util.logging.aspectj;

import java.util.logging.Level;

import com.ibm.bbp.util.logging.BBPLogger;
import com.ibm.bbp.util.logging.BBPLoggerFactory;

/**
 * Abstract aspect used to simplify the application of logging to classes.
 * 
 * To define logging using this framework, create a new aspect which extends this aspect, and define the abstract
 * getConcreteLogger method and abstract loggedClasses pointcut.
 */
public aspect CommonLogging {

	private volatile static BBPLogger logger = BBPLoggerFactory.getLogger();

	protected static BBPLogger getLogger() {
		return logger;
	}

	public pointcut shouldLog() : if(logger.isLoggable(Level.FINEST)) && !ignoreClasses() && !ignoreMethods();

	// List the classes there that should be LOGGED WITH ARGS
	public pointcut loggedClassesPrintArgs():
                                              // by default aspect everything.
                                              // remove classes by adding to the NoArgs() and ignore() lists
                                                // below
                                              within(com.ibm.bbp..*);

	// List the classes there that should be LOGGED BUT W/O ANY ARGS
	// This list will trump anything defined in loggedClassesPrintArgs
	public pointcut loggedClassesNoArgs(): 
                        within(com.ibm.bbp.util.Password) || 
                        execution(* setPassword(..)) ||
                        execution(* getPassword(..)) ||
                        execution(* trimEqualsIgnoreCase(..)) ||
                        execution(* trimUtil(..)) ||
                        execution(* setConfirmPassword(..)) ||
                        execution(* getConfirmPassword(..)) ||
                        execution(* getNewPassword(..)) ||
                        execution(* setNewPassword(..)) ||
                        execution(* getPassWd(..)) ||
                        execution(* setPassWd(..)) ||
                        execution(* getIvrRecordingPasscode(..)) ||
                        execution(* setIvrRecordingPasscode(..)) ||
                        execution(* getModeratorPasscode(..)) ||
                        execution(* setModeratorPasscode(..)) ||
                        execution(* getPasscode(..)) ||
                        execution(* setPasscode(..)) ||
                        execution(* getVoipConfirmPasscode(..)) ||
                        execution(* setVoipConfirmPasscode(..)) ||
                        execution(* setExtensionPassword(..)) ||
                        execution(* getExtensionPassword(..)) ||
                        execution(* getParticipant(..)) ||
                        execution(* setParticipant(..)) ||
                        execution(* getModerator(..)) ||
                        execution(* setModerator(..)) ||
                        execution(* getCurrentPassword(..)) ||
                        execution(* setCurrentPassword(..));

	// List the classes there that should NEVER BE LOGGED
	// This list will trump anything defined in loggedClassesPrintArgs and loggedClassesNoArgs
	public pointcut ignoreClasses():       within(com.ibm.bbp.util.logging..*) || 
                                           within(com.ibm.bbp.util.exception..*) ||
                                           within(com.ibm.bbp.util.resources..*) ||
                                           within(com.ibm.bbp.util.os..*) || // causes a loop b/c os makes a
                                                                                // call
                                           within(com.ibm.bbp.util.BBPVersion) || // causes loop with logging
                                           within(com.ibm.bbp.util.BBPTrial) || 
                                           within(com.ibm.bbp.framework.common.AutoJSON) ||
                                           within(com.ibm.bbp.framework.common.AutoJSONBase+) ||
                                           within(com.ibm.bbp.fix..*) || // fix needs to be refactored
                                           within(com.ibm.bbp.desktop.ui.Activator) || // Activators are early in plug-in life and
                                                                        // likely have not init logging
                                           within(com.ibm.bbp.framework.server.BbpSafServerLogger) ||
										   within(com.ibm.bbp.framework.common.BbpSafLogger) || // need to initialize logging
                                           within(com.ibm.bbp.gui.people.RemovePeopleBean); // locks up ISC
	public pointcut ignoreMethods():    execution(* toString());

	// Method pointcuts
	pointcut enterConstructorNoArgs() : loggedClassesNoArgs()
                                     && shouldLog()
                                     && execution(new (..));

	pointcut enterConstructorPrintArgs() : loggedClassesPrintArgs()
                                        && !loggedClassesNoArgs()
                                        && shouldLog()
                                        && execution(new (..));

	pointcut enterMethodNoArgs() : loggedClassesNoArgs()
                                   && shouldLog()
                                   && execution(* * (..));

	pointcut enterMethodPrintArgs() : loggedClassesPrintArgs()
                                   && !loggedClassesNoArgs()
                                   && shouldLog()
                                   && execution(* * (..));

	pointcut exceptions() : shouldLog() &&
							(loggedClassesNoArgs() || loggedClassesPrintArgs());

	/**
	 * For all exception blocks, log using the log factory
	 */
	before(Exception e) : exceptions() &&
                          handler(*) &&
                          args(e) {

		try {
			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append(new String[] { e.getLocalizedMessage() });

			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}

			myLogger.traceFinest(logMessage.toString(), getClassName(oThis), thisJoinPointStaticPart.getSignature()
					.toString());

		} catch (Throwable ex) {
			System.err.println("Exception thrown in aspect : [before(Exception e)] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + ex);
			ex.printStackTrace();
		}
	}

	/**
	 * For all constructors, log using the log factory but do not print out arguments
	 */
	before() : enterConstructorNoArgs() {

		try {

			BBPLogger myLogger = getLogger();

			StringBuffer logMessage = new StringBuffer();
			logMessage.append("-->");
			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}
			myLogger.traceFinest(logMessage.toString(), getClassName(oThis), thisJoinPointStaticPart.getSignature()
					.toString());

		} catch (Throwable e) {
			System.err.println("Exception thrown in aspect : [before() : enterConstructorNoArgs()] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * For all constructors, log using the log factory and include the arguments
	 */
	before() : enterConstructorPrintArgs() {

		try {
			// check to see if logging enabled before
			// creating the string

			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append("-->");
			Object[] arguments = null;
			if (thisJoinPoint != null) {
				arguments = thisJoinPoint.getArgs();
			}
			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}

			myLogger.traceFinest(logMessage.toString(), arguments, getClassName(oThis), thisJoinPointStaticPart
					.getSignature().toString());

		} catch (Throwable e) {
			System.err.println("Exception thrown in aspect : [before() : enterConstructorPrintArgs()] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * Before all methods, log entry using the log factory but do not print the arguments Do not log methods called on
	 * persistant agent thread.
	 */
	before() : enterMethodNoArgs() {
		try {

			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append("-->");
			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}

			myLogger.traceFinest(logMessage.toString(), getClassName(oThis), thisJoinPointStaticPart.getSignature()
					.toString());
		} catch (Throwable e) {
			System.err.println("Exception thrown in aspect : [before() : enterMethodNoArgs()] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * Before all methods, log entry using the log factory and include the arguments
	 */
	before() : enterMethodPrintArgs() {

		try {
			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append("-->");
			Object[] arguments = null;
			if (thisJoinPoint != null) {
				arguments = thisJoinPoint.getArgs();
			}

			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}

			myLogger.traceFinest(logMessage.toString(), arguments, getClassName(oThis), thisJoinPointStaticPart
					.getSignature().toString());
		} catch (Throwable e) {
			System.err.println("Exception thrown in aspect : [before() : enterMethodPrintArgs()] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * After all methods, log exit using the log factory but do not print the arguments
	 */
	after() returning(Object obj) : enterMethodNoArgs() {
		try {

			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append("<--");
			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}
			myLogger.traceFinest(logMessage.toString(), getClassName(oThis), thisJoinPointStaticPart.getSignature()
					.toString());
		} catch (Throwable e) {
			System.err.println("Exception thrown in aspect : [after() returning(Object obj) : enterMethodNoArgs()] : "
					+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * After all methods, log exit using the log factory
	 */
	after() returning(Object obj) : enterMethodPrintArgs() {
		try {
			BBPLogger myLogger = getLogger();
			StringBuffer logMessage = new StringBuffer();
			logMessage.append("<-- (" + obj);
			logMessage.append(")");

			Object oThis = null;
			if (thisJoinPoint != null) {
				oThis = thisJoinPoint.getThis();
			}
			myLogger.traceFinest(logMessage.toString(), getClassName(oThis), thisJoinPointStaticPart.getSignature()
					.toString());
		} catch (Throwable e) {
			System.err
					.println("Exception thrown in aspect : [after() returning(Object obj) : enterMethodPrintArgs()] : "
							+ thisJoinPointStaticPart.getSignature() + " : " + e);
			e.printStackTrace();
		}
	}

	/**
	 * Gets the clas name of the given object or else returns null
	 */
	protected String getClassName(Object clas) {
		if (clas == null) {
			return null;
		}
		Class c = clas.getClass();
		if (c == null)
			return null;
		return c.getName();
	}

}
